//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __MakeTriggers__
#define __MakeTriggers__

#include "H5Cpp.h"
#include "Streams.h"
#include "Triggers.h"

using namespace std;

/**
 * Write triggers on disk.
 * This class was designed to write triggers in a ROOT tree following the <a href="../../Main/convention.html#triggers">GWOLLUM convention</a> for triggers (some other output formats are also available). When triggers are produced by some external algorithms, the MakeTriggers class can be used to manage output triggers, to save them and to perform some basic operations (like clustering).
 * 
 * The user is responsible to fill both the Segments and the Triggers structure from which this class inherits. In particular, one must make sure that the segments and the triggers match the <a href="../../Main/convention.html#triggers">GWOLLUM convention</a> for triggers.
 * \author    Florent Robinet
 */
class MakeTriggers: public Triggers, public Segments , public Streams {

 public:

  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the MakeTriggers class.
   * The different structures are initialized: the input stream, the trigger tree, the segments, the cluster tree and the metadata tree.
   * @param aStreamName stream name
   * @param aVerbose verbosity level
   */
  MakeTriggers(const string aStreamName, const int aVerbose=0);

  /**
   * Destructor of the MakeTriggers class.
   */
  virtual ~MakeTriggers(void);
  /**
     @}
  */
  
  /**
   * Resets all.
   * the memory is cleaned up:
   * - triggers/clusters (Triggers::ResetTriggers())
   * - segments
   * - metadata
   */
  void Reset(void);
  /**
     @}
  */
  
  /**
   * Set the process name for the meta-data field 'Mprocessname'.
   * See <a href="../../Main/convention.html#triggers">GWOLLUM convention</a>.
   * By default Mprocessname = "PROC".
   * @param aMprocessname process name
   */
  inline void SetMprocessname(const string aMprocessname){ Mprocessname=aMprocessname; };

  /**
   * Sets the version for this process.
   * By default = "0".
   * @param aMprocessversion process version
   */
  inline void SetProcessVersion(const string aMprocessversion){ Mprocessversion=aMprocessversion; };

  /**
   * Initializes the user-defined meta-data fields.
   * 'aMusername' should be a non-empty vector with the list of meta-data names. 'aMuservalue' should be a vector with the list of meta-data types (same size as aMusername). Supported types: "d" for double, "i" for int, "s" for string.
   * Default values are -1 for double and int types and "none" for string type.
   * @param aMusername vector of metadata names
   * @param aMusertype vector of metadata types (i/d/s)
   */
  bool InitUserMetaData(vector<string> aMusername, vector<string> aMusertype);

  /**
   * Set user-defined meta-data.
   * Set the meta-data named 'aMusername' with the value 'aMuservalue'.
   * @param aMusername metadata name
   * @param aMuservalue metadata value
   */
  bool SetUserMetaData(const string aMusername, const double aMuservalue);

  /**
   * Set user-defined meta-data.
   * Set the meta-data named 'aMusername' with the value 'aMuservalue'.
   * @param aMusername metadata name
   * @param aMuservalue metadata value
   */
  bool SetUserMetaData(const string aMusername, const int aMuservalue);

  /**
   * Set user-defined meta-data.
   * Set the meta-data named 'aMusername' with the value 'aMuservalue'.
   * @param aMusername metadata name
   * @param aMuservalue metadata value
   */
  bool SetUserMetaData(const string aMusername, const string aMuservalue);  

  /**
   * Writes triggers on disk.
   * This function writes all the triggers added until now in an output file as well as the segments and metadata trees. The MakeTriggers object is then completely reset (see Triggers::ResetTriggers()).
   *
   * The output file path is returned. In case of several formats, root then txt then xml file is returned. "none" is returned if this function fails in some way. Several output formats are supported:
   * - 'root': standard GWOLLUM root trigger files
   * - 'hdf5': standard hdf5 format
   * - 'xml': LIGO xml sngl_burst table
   * - 'txt': text file
   *
   * Multiple output formats can be requested. Use a single string to list the requested formats. Note that for the hdf5, xml and txt formats, if triggers were clustered, only clusters are saved. For the root format, only triggers are saved.
   *
   * The optional argument 'aFileName' is used to bypass the file naming convention of GWOLLUM. With this option, the trigger file will be given a new name: [aFileName].[format]
   *
   * For ROOT files, it is possible to write the trees in a ROOT sub-directory. Use the optional argument 'aSubDir'. Only in this case, the trigger file can be updated when adding several directories. To do this, use aWriteMode="UPDATE".
   * @param aOutDir output directory, must exist
   * @param aFileFormat list of file formats
   * @param aFileName new base name for ouput file
   * @param aSubDir ROOT sub-directory name to write the triggers (only for ROOT format)
   * @param aWriteMode write mode: "RECREATE" or "UPDATE" (only with sub-directories)
   * @param aWriteMeta flag to write (or not) the metadata tree
   */
  string Write(const string aOutDir=".",
	       const string aFileFormat="root",
	       const string aFileName="default",
	       const string aSubDir="",
	       const string aWriteMode="RECREATE",
	       const bool aWriteMeta=true);

 protected:

  // METADATA
  TTree *Mtree;               ///< metadata tree
  double Mstart,              ///< meta start time
    Mend;                     ///< meta end time
  string Mprocessname,        ///< meta process name
    Mstreamname,              ///< meta stream name
    Mprocessversion,          ///< meta process version
    Mprocessuser;             ///< meta user name
  double Mfmin,               ///< meta minimum fstart
    Mfmax,                    ///< meta maximum fend
    Mqmin,                    ///< meta minimum Q
    Mqmax,                    ///< meta maximum Q
    Msnrmin,                  ///< meta minimum SNR
    Msnrmax;                  ///< meta maximum SNR
  vector <string> Musername;  ///< user metadata name
  vector <string> Musertype;  ///< user metadata type
  double *Mdvar;              ///< user metatree var - double
  int    *Mivar;              ///< user metatree var - int
  string *Msvar;              ///< user metatree var - string
 
 private:
  
  bool WriteHDF5(const string hdf5filename);
  bool WriteTXT(const string txtfilename);
  bool WriteXML(const string xmlfilename);

  typedef struct h5_triggers{
    double h5_time;
    double h5_freq;
    double h5_snr;
    double h5_q;
    double h5_amp;
    double h5_ph;
    double h5_tstart;
    double h5_tend;
    double h5_fstart;
    double h5_fend;
  }h5_triggers; ///< trigger structure for hdf5

  typedef struct h5_segments{
    double h5_start;
    double h5_end;
  }h5_segments; ///< segments structure for hdf5

  H5::CompType *h5_type; ///< datatype for hdf5 (triggers)
  H5::CompType *h5_type_s; ///< datatype for hdf5 (segments)

  ClassDef(MakeTriggers,0)  
};

#endif


