//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __ReadTriggerMetaData__
#define __ReadTriggerMetaData__

#include "ReadTriggerSegments.h"

using namespace std;


/**
 * Access meta-data in trigger files.
 * This class was designed to read triggers meta-data stored in TTrees saved in ROOT files (following the <a href="../../Main/convention.html#triggers">GWOLLUM convention</a>). It should be used when you're only interested by the meta-data content of trigger files. Only the segment and meta-data content of the files is loaded so this class is much faster than the ReadTriggers class since the triggers are never loaded. This is quite useful when hundreds of files need to be scanned to access the trigger meta-characteristics.
 * \author    Florent Robinet
 */
class ReadTriggerMetaData : public ReadTriggerSegments {

 public:

  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the ReadTriggerMetaData class.
   * The metadata in files designated by the file pattern 'aPattern' are loaded. The metadata tree is accessed in the root directory 'aDirectory'. The verbosity level, set by 'aVerbose', follows the <a href="../../Main/convention.html#verbosity">GWOLLUM convention</a> for verbosity.
   *
   * @param aPattern input file pattern
   * @param aDirectory trigger ROOT directory
   * @param aVerbose verbosity level
   */
  ReadTriggerMetaData(const string aPattern, const string aDirectory="", const int aVerbose=0);

  /**
   * Destructor of the ReadTriggerMetaData class.
   */
  virtual ~ReadTriggerMetaData(void);
  /**
     @}
  */
  
  /**
   * Returns the process name.
   * Returns "none" if the information is missing.
   */
  inline string GetProcessName(void){ 
    if(status_OK) return Mprocessname;
    return "none";
  };
  
  /**
   * Returns the stream name.
   */
  inline string GetStreamName(void){
    if(status_OK) return Mstreamname;
    return "none";
  };

  /**
   * Returns the lower end of the trigger frequency range.
   * Returns 1e20 if the information is missing.
   */
  inline double GetFrequencyMin(void){ return Mfmin_stat; };

  /**
   * Returns the upper end of the trigger frequency range.
   * Returns -1.0 if the information is missing.
   */
  inline double GetFrequencyMax(void){ return Mfmax_stat; };

  /**
   * Returns the lower end of the trigger Q range.
   * Returns 1e20 if the information is missing.
   */
  inline double GetQMin(void){ return Mqmin_stat; };

  /**
   * Returns the upper end of the trigger Q range.
   * Returns -1.0 if the information is missing.
   */
  inline double GetQMax(void){ return Mqmax_stat; };

  /**
   * Returns the lower end of the trigger SNR range.
   * Returns 1e20 if the information is missing.
  */
  inline double GetSNRMin(void){ return Msnrmin_stat; };

  /**
   * Returns the upper end of the trigger SNR range.
   * Returns -1.0 if the information is missing.
   */
  inline double GetSNRMax(void){ return Msnrmax_stat; };
 
  /**
   * Merges the trigger files into one.
   * Returns the name of the merged trigger file. Returns "none" if this function fails.
   * @param aOutdir output directory
   */
  string Merge(const string aOutdir=".");

 private:
  
  // DATA
  double *Mstart,          ///< start time
    *Mend;                 ///< end time
  string Mprocessname;     ///< process name
  string Mstreamname;      ///< stream name
  double Mfmin_stat,       ///< meta frequency min (static)
    Mfmax_stat,            ///< meta frequency max (static)
    Mqmin_stat,            ///< meta Q min (static)
    Mqmax_stat,            ///< meta Q max (static)
    Msnrmin_stat,          ///< meta SNR min (static)
    Msnrmax_stat;          ///< meta SNR max (static)

  double *Mfmin,           ///< meta frequency min
    *Mfmax,                ///< meta frequency max
    *Mqmin,                ///< meta Q min
    *Mqmax,                ///< meta Q max
    *Msnrmin,              ///< meta SNR min
    *Msnrmax;              ///< meta SNR max

  bool GetInputMetaData(void); ///< get input metadata from files


  ClassDef(ReadTriggerMetaData,0)  
};

#endif


